@@ -203,6 +203,12 @@ void KWordGraphicsHandler::init(Drawings * pDrawings, const wvWare::Word97::FIB
     m_drawings = pDrawings;
     m_fib = const_cast<wvWare::Word97::FIB *>(&fib);
 
+    //provide the backgroud color information to the Document
+    DrawStyle ds = getDrawingStyle();
+    MSO::OfficeArtCOLORREF fc = ds.fillColor();
+    QColor color(fc.red, fc.green, fc.blue);
+    m_document->addBgColor(color.name());
+
     return;
 }
 
