@@ -29,25 +29,26 @@
 
 namespace KSpread
 {
-class Doc;
+class CalculationSettings;
 class Value;
 class ValueConverter;
 
 /**
+ * \ingroup Value
  * Generates a textual representation of a Value with a given formatting.
  */
-class ValueFormatter
+class KSPREAD_TEST_EXPORT ValueFormatter
 {
 public:
     /**
      * Constructor.
      */
-    explicit ValueFormatter( const ValueConverter* converter );
+    explicit ValueFormatter(const ValueConverter* converter);
 
     /**
-     * Returns the document this ValueFormatter belongs to.
+     * Returns the calculation settings this ValueFormatter uses.
      */
-    const Doc* doc() const;
+    const CalculationSettings* settings() const;
 
     /**
      * Creates a textual representation of \p value with the explicit given
@@ -64,19 +65,25 @@ public:
                      Style::FloatFormat floatFormat = Style::OnlyNegSigned,
                      const QString& prefix = QString(),
                      const QString& postfix = QString(),
-                     const QString& currencySymbol = QString());
+                     const QString& currencySymbol = QString(), const QString& formatString = QString());
 
     /**
      * Creates a date format.
      * \param formatType the value format, e.g. number, date
      */
-    QString dateFormat( const QDate& date, Format::Type formatType );
+    QString dateFormat(const QDate& date, Format::Type formatType, const QString& formatString = QString() );
 
     /**
      * Creates a time format.
      * \param formatType the value format, e.g. number, date
      */
-    QString timeFormat( const QDateTime& time, Format::Type formatType );
+    QString timeFormat(const QDateTime& time, Format::Type formatType, const QString& formatString = QString() );
+
+    /**
+     * Creates a date and time format.
+     * \param formatType the value format, e.g. number, date
+     */
+    QString dateTimeFormat(const QDateTime& time, Format::Type formatType, const QString& formatString = QString() );
 
 protected:
     /**
@@ -84,7 +91,7 @@ protected:
      * in a cell with a given format type
      * \param formatType the value format, e.g. number, date
      */
-    Format::Type determineFormatting( const Value& value, Format::Type formatType );
+    Format::Type determineFormatting(const Value& value, Format::Type formatType);
 
     /**
      * Creates a number format.
@@ -93,16 +100,17 @@ protected:
      * \param floatFormat the number format, i.e. signed/unsigned information
      * \param currencySymbol the currency symbol
      */
-    QString createNumberFormat( Number value, int precision,
-                                Format::Type formatType,
-                                Style::FloatFormat floatFormat,
-                                const QString& currencySymbol );
+    QString createNumberFormat(Number value, int precision,
+                               Format::Type formatType,
+                               Style::FloatFormat floatFormat,
+                               const QString& currencySymbol,
+                               const QString& formatString);
 
     /**
      * Creates a fraction format.
      * \param formatType the value format, e.g. number, date
      */
-    QString fractionFormat( Number value, Format::Type formatType );
+    QString fractionFormat(Number value, Format::Type formatType);
 
     /**
      * Creates a complex number format.
@@ -111,17 +119,17 @@ protected:
      * \param floatFormat the number format, i.e. signed/unsigned information
      * \param currencySymbol the currency symbol
      */
-    QString complexFormat( const Value& value, int precision,
-                           Format::Type formatType,
-                           Style::FloatFormat floatFormat,
-                           const QString& currencySymbol );
+    QString complexFormat(const Value& value, int precision,
+                          Format::Type formatType,
+                          Style::FloatFormat floatFormat,
+                          const QString& currencySymbol);
 
     /**
      * Removes the trailing zeros and the decimal symbol \p decimalSymbol in
      * \p string , if necessary.
      * \return the truncated string
      */
-    QString removeTrailingZeros( const QString& string, const QString& decimalSymbol );
+    QString removeTrailingZeros(const QString& string, const QString& decimalSymbol);
 
 private:
     const ValueConverter* m_converter;
