@@ -20,12 +20,15 @@
 
 #include "KoShapeGroup.h"
 #include "KoShapeContainerModel.h"
+#include "KoShapeLayer.h"
 #include "SimpleShapeContainerModel.h"
 #include "KoShapeSavingContext.h"
 #include "KoShapeLoadingContext.h"
 #include "KoXmlWriter.h"
 #include "KoXmlReader.h"
 #include "KoShapeRegistry.h"
+#include "KoShapeBorderModel.h"
+#include "KoShapeShadow.h"
 
 #include <QPainter>
 
@@ -35,6 +38,10 @@ KoShapeGroup::KoShapeGroup()
     setSize(QSizeF(0, 0));
 }
 
+KoShapeGroup::~KoShapeGroup()
+{
+}
+
 void KoShapeGroup::paintComponent(QPainter &painter, const KoViewConverter &converter)
 {
     Q_UNUSED(painter);
@@ -47,8 +54,9 @@ bool KoShapeGroup::hitTest(const QPointF &position) const
     return false;
 }
 
-void KoShapeGroup::childCountChanged()
+void KoShapeGroup::shapeCountChanged()
 {
+    // TODO: why is this needed here ? the group/ungroup command should take care of this
     QRectF br = boundingRect();
     setAbsolutePosition(br.topLeft(), KoFlake::TopLeftCorner);
     setSize(br.size());
@@ -60,7 +68,7 @@ void KoShapeGroup::saveOdf(KoShapeSavingContext & context) const
     saveOdfAttributes(context, (OdfMandatories ^ OdfLayer) | OdfAdditionalAttributes);
     context.xmlWriter().addAttributePt("svg:y", position().y());
 
-    QList<KoShape*> shapes = iterator();
+    QList<KoShape*> shapes = this->shapes();
     qSort(shapes.begin(), shapes.end(), KoShape::compareShapeZIndex);
 
     foreach(KoShape* shape, shapes) {
@@ -76,16 +84,31 @@ bool KoShapeGroup::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &
     loadOdfAttributes(element, context, OdfMandatories | OdfAdditionalAttributes | OdfCommonChildElements);
 
     KoXmlElement child;
+    QMap<KoShapeLayer*, int> usedLayers;
     forEachElement(child, element) {
         KoShape * shape = KoShapeRegistry::instance()->createShapeFromOdf(child, context);
         if (shape) {
-            addChild(shape);
+            KoShapeLayer *layer = dynamic_cast<KoShapeLayer*>(shape->parent());
+            if (layer) {
+                usedLayers[layer]++;
+            }
+            addShape(shape);
         }
     }
+    KoShapeLayer *parent = 0;
+    int maxUseCount = 0;
+    // find most used layer and use this as parent for the group
+    for (QMap<KoShapeLayer*, int>::const_iterator it(usedLayers.constBegin()); it != usedLayers.constEnd(); ++it) {
+        if (it.value() > maxUseCount) {
+            maxUseCount = it.value();
+            parent = it.key();
+        }
+    }
+    setParent(parent);
 
     QRectF bound;
     bool boundInitialized = false;
-    foreach(KoShape * shape, iterator()) {
+    foreach(KoShape * shape, shapes()) {
         if (! boundInitialized) {
             bound = shape->boundingRect();
             boundInitialized = true;
@@ -96,8 +119,37 @@ bool KoShapeGroup::loadOdf(const KoXmlElement & element, KoShapeLoadingContext &
     setSize(bound.size());
     setPosition(bound.topLeft());
 
-    foreach(KoShape * shape, iterator())
-    shape->setAbsolutePosition(shape->absolutePosition() - bound.topLeft());
+    foreach(KoShape * shape, shapes())
+        shape->setAbsolutePosition(shape->absolutePosition() - bound.topLeft());
 
     return true;
 }
+
+void KoShapeGroup::shapeChanged(ChangeType type, KoShape *shape)
+{
+    Q_UNUSED(shape);
+    switch (type) {
+    case KoShape::BorderChanged:
+    {
+        KoShapeBorderModel *stroke = border();
+        if (stroke) {
+            if (stroke->deref())
+                delete stroke;
+            setBorder(0);
+        }
+        break;
+    }
+    case KoShape::ShadowChanged:
+    {
+        KoShapeShadow *shade = shadow();
+        if (shade) {
+            if (shade->deref())
+                delete shade;
+            setShadow(0);
+        }
+        break;
+    }
+    default:
+        break;
+    }
+}
