@@ -23,7 +23,7 @@
 #include "KoText.h"
 #include "kotext_export.h"
 
-#include <KoShapeUserData.h>
+#include <KoTextShapeDataBase.h>
 #include <KoInsets.h>
 #include <KoXmlReaderForward.h>
 
@@ -44,7 +44,7 @@ class KoDocumentRdfBase;
  * multiple instances of this shape data) can share one QTextDocument by providing a
  * different view on (a different part of) the QTextDocument.
  */
-class KOTEXT_EXPORT KoTextShapeData : public KoShapeUserData
+class KOTEXT_EXPORT KoTextShapeData : public KoTextShapeDataBase
 {
     Q_OBJECT
 public:
@@ -59,8 +59,6 @@ public:
      *    of this data and the doc will be deleted when this shapeData dies.
      */
     void setDocument(QTextDocument *document, bool transferOwnership = true);
-    /// return the document
-    QTextDocument *document();
 
     /**
      * return the amount of points into the document (y) this shape will display.
@@ -110,18 +108,6 @@ public:
     /// emits a relayout
     void fireResizeEvent();
 
-    /**
-     * Set the margins that will make the shapes text area smaller.
-     * The shape that owns this textShapeData object will layout text in an area
-     * confined by the shape size made smaller by the margins set here.
-     * @param margins the margins that shrink the text area.
-     */
-    void setShapeMargins(const KoInsets &margins);
-    /**
-     * returns the currently set margins for the shape.
-     */
-    KoInsets shapeMargins() const;
-
     /**
      * Calling this method will do a layout run of the text for this shape using the
      * provided textPage. The currently set page() will not be touched.
@@ -145,12 +131,29 @@ public:
     * @see the @a KoTextLoader::loadBody() method which got called by this method
     * to load the ODF.
     */
-    bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext & context, KoDocumentRdfBase *rdfData = 0);
+    bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context, KoDocumentRdfBase *rdfData, KoShape *shape = 0);
+
+    /**
+    * Load the TextShape from ODF.
+    * Overloaded method provided for your convenience.
+    */
+    virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context) {
+        return loadOdf(element, context, 0);
+    }
+
     /**
     * Store the TextShape data as ODF.
     * @see TextShape::saveOdf()
     */
-    void saveOdf(KoShapeSavingContext & context, int from = 0, int to = -1, KoDocumentRdfBase *rdfData = 0) const;
+    void saveOdf(KoShapeSavingContext &context, KoDocumentRdfBase *rdfData, int from = 0, int to = -1) const;
+
+    /**
+    * Store the TextShape data as ODF.
+    * Overloaded method provided for your convenience.
+    */
+    virtual void saveOdf(KoShapeSavingContext &context, int from = 0, int to  = -1) const {
+        saveOdf(context, 0, from, to);
+    }
 
     /**
      * Set the page direction.
@@ -165,11 +168,6 @@ public:
      */
     KoText::Direction pageDirection() const;
 
-    /** Sets the vertical alignment of all the text inside the shape. */
-    void setVerticalAlignment(Qt::Alignment alignment);
-    /** Returns the vertical alignment of all the text in the shape */
-    Qt::Alignment verticalAlignment() const;
-
 signals:
     /**
      * emitted when the shape thinks it should be relayouted, for example after
@@ -180,7 +178,7 @@ signals:
     void relayout();
 
 private:
-    KoTextShapeDataPrivate *const d;
+    Q_DECLARE_PRIVATE(KoTextShapeData)
 };
 
 #endif
