@@ -23,6 +23,7 @@
 #include <QPainter>
 #include <QPixmap>
 #include <KoColorConversions.h>
+#include <QTimer>
 
 enum CurrentHandle {
     NoHandle,
@@ -53,6 +54,8 @@ struct KoTriangleColorSelector::Private {
     CurrentHandle handle;
     qreal triangleHandleSize;
     bool invalidTriangle;
+    int lastX, lastY;
+    QTimer updateTimer;
 };
 
 KoTriangleColorSelector::KoTriangleColorSelector(QWidget* parent) : QWidget(parent), d(new Private)
@@ -66,6 +69,11 @@ KoTriangleColorSelector::KoTriangleColorSelector(QWidget* parent) : QWidget(pare
     setMouseTracking( true );
     updateTriangleCircleParameters();
     d->invalidTriangle = true;
+    d->lastX = -1;
+    d->lastY = -1;
+    d->updateTimer.setInterval(1);
+    d->updateTimer.setSingleShot(true);
+    connect(&(d->updateTimer), SIGNAL(timeout()), this, SLOT(update()));
 }
 
 KoTriangleColorSelector::~KoTriangleColorSelector()
@@ -149,7 +157,7 @@ void KoTriangleColorSelector::setHue(int h)
     d->hue = h;
     tellColorChanged();
     d->invalidTriangle = true;
-    update();
+    d->updateTimer.start();
 }
 
 int KoTriangleColorSelector::value() const
@@ -163,7 +171,7 @@ void KoTriangleColorSelector::setValue(int v)
     d->value = v;
     tellColorChanged();
     d->invalidTriangle = true;
-    update();
+    d->updateTimer.start();
 }
 
 int KoTriangleColorSelector::saturation() const
@@ -177,7 +185,7 @@ void KoTriangleColorSelector::setSaturation(int s)
     d->saturation = s;
     tellColorChanged();
     d->invalidTriangle = true;
-    update();
+    d->updateTimer.start();
 }
 
 void KoTriangleColorSelector::setHSV(int h, int s, int v)
@@ -185,12 +193,12 @@ void KoTriangleColorSelector::setHSV(int h, int s, int v)
     h = qBound(0, h, 360);
     s = qBound(0, s, 255);
     v = qBound(0, v, 255);
+    d->invalidTriangle = (d->hue != h);
     d->hue = h;
     d->value = v;
     d->saturation = s;
     tellColorChanged();
-    d->invalidTriangle = true;
-    update();
+    d->updateTimer.start();
 }
 
 QColor KoTriangleColorSelector::color() const
@@ -209,7 +217,7 @@ void KoTriangleColorSelector::setQColor(const QColor& c)
         if( hue >= 0 && hue <= 360)
             d->hue = hue;
         d->invalidTriangle = true;
-        update();
+        d->updateTimer.start();
     }
 }
 
@@ -239,18 +247,19 @@ void KoTriangleColorSelector::generateTriangle()
     // Length of triangle
     int hue_ = hue();
     
-    for(int y = 0; y < d->sizeColorSelector; y++)
+    for(int y = 0; y < d->sizeColorSelector; ++y)
     {
         qreal ynormalize = ( d->triangleTop - y ) / ( d->triangleTop - d->triangleBottom );
         qreal v = 255 * ynormalize;
         qreal ls_ = (ynormalize) * d->triangleLength;
         qreal startx_ = d->centerColorSelector - 0.5 * ls_;
-        for(int x = 0; x < d->sizeColorSelector; x++)
+        uint* data = reinterpret_cast<uint*>(image.scanLine(y));
+        for(int x = 0; x < d->sizeColorSelector; ++x, ++data)
         {
             qreal s = 255 * (x - startx_) / ls_;
             if(v < -1.0 || v > 256.0 || s < -1.0 || s > 256.0 )
             {
-                image.setPixel(x,y, qRgba(0,0,0,0));
+                *data = qRgba(0,0,0,0);
             } else {
                 qreal va = 1.0, sa = 1.0;
                 if( v < 0.0) { va = 1.0 + v; v = 0; }
@@ -262,9 +271,9 @@ void KoTriangleColorSelector::generateTriangle()
                 qreal coef = va * sa;
                 if( coef < 0.999)
                 {
-                    image.setPixel(x,y, qRgba( (int)(r * coef), (int)(g * coef), (int)(b * coef), (int)(255 * coef)));
+                    *data = qRgba( (int)(r * coef), (int)(g * coef), (int)(b * coef), (int)(255 * coef));
                 } else {
-                    image.setPixel(x,y, qRgba(r, g, b, 255 ));
+                    *data = qRgba(r, g, b, 255 );
                 }
             }
         }
@@ -340,6 +349,14 @@ void KoTriangleColorSelector::mouseMoveEvent( QMouseEvent * event )
 void KoTriangleColorSelector::selectColorAt(int _x, int _y, bool checkInWheel)
 {
     Q_UNUSED( checkInWheel );
+    
+    if (d->lastX == _x && d->lastY == _y)
+    {
+        return;
+    }
+    d->lastX = _x;
+    d->lastY = _y;
+    
     qreal x = _x - 0.5*width();
     qreal y = _y - 0.5*height();
     // Check if the click is inside the wheel
@@ -348,7 +365,7 @@ void KoTriangleColorSelector::selectColorAt(int _x, int _y, bool checkInWheel)
          || d->handle == HueHandle ) {
         d->handle = HueHandle;
         setHue( (int)(atan2(y, x) * 180 / M_PI ) + 180);
-        update();
+        d->updateTimer.start();
     }
     else {
     // Compute the s and v value, if they are in range, use them
@@ -369,7 +386,7 @@ void KoTriangleColorSelector::selectColorAt(int _x, int _y, bool checkInWheel)
                 setHSV( d->hue, sat * 255, ynormalize * 255);
             }
         }
-        update();
+        d->updateTimer.start();
     }
 }
 
