@@ -31,21 +31,31 @@ namespace Libemf
 //                         Class OutputPainterStrategy
 
 
-OutputPainterStrategy::OutputPainterStrategy() :
-    m_header( 0 ),
-    m_path( 0 ), 
-    m_currentlyBuildingPath( false ), 
-    m_image( 0 ), 
-    m_currentCoords()
+OutputPainterStrategy::OutputPainterStrategy()
+    : m_header( 0 )
+    , m_path( 0 )
+    , m_currentlyBuildingPath( false )
+    , m_image( 0 )
+      // FIXME: Initialize all vars here!
+    , m_mapMode(MM_TEXT)
+    , m_textAlignMode(TA_NOUPDATECP) // == TA_TOP == TA_LEFT
+    , m_currentCoords()
 {
+    m_painter         = 0;
+    m_painterSaves    = 0;
+    m_outputSize      = QSize();
+    m_keepAspectRatio = true;
 }
 
-OutputPainterStrategy::OutputPainterStrategy(QPainter &painter, QSize &size, 
+OutputPainterStrategy::OutputPainterStrategy(QPainter &painter, QSize &size,
                                              bool keepAspectRatio)
     : m_header( 0 )
     , m_path( 0 )
     , m_currentlyBuildingPath( false )
     , m_image( 0 )
+      // FIXME: Initialize all vars here!
+    , m_mapMode(MM_TEXT)
+    , m_textAlignMode(TA_NOUPDATECP) // == TA_TOP == TA_LEFT
     , m_currentCoords()
 {
     m_painter         = &painter;
@@ -57,7 +67,7 @@ OutputPainterStrategy::OutputPainterStrategy(QPainter &painter, QSize &size,
 OutputPainterStrategy::~OutputPainterStrategy()
 {
     delete m_header;
-    //delete m_painter;
+    //delete m_painter;    // Don't delete the painter because we don't own it.
     delete m_path;
     delete m_image;
 }
@@ -83,9 +93,10 @@ void OutputPainterStrategy::init( const Header *header )
     // To be precise, it seems that the StretchDiBits record uses the
     // physical size (stored in header.frame()) to specify where to
     // draw the picture rather than logical coordinates.
+    // (This has since been doubted, and is currently disabled.)
     m_header = new Header(*header);
 
-    QSize  outputSize = header->bounds().size();
+    QSize  headerBoundsSize = header->bounds().size();
 
 #if DEBUG_EMFPAINT
     kDebug(31000) << "----------------------------------------------------------------------";
@@ -96,7 +107,7 @@ void OutputPainterStrategy::init( const Header *header )
     kDebug(31000) << "outputSize           =" << m_outputSize.width() << m_outputSize.height();
 
     kDebug(31000) << "Device =" << header->device().width() << header->device().height();
-    kDebug(31000) << "Millimeters =" << header->millimeters().width() 
+    kDebug(31000) << "Millimeters =" << header->millimeters().width()
                   << header->millimeters().height();
 #endif
 
@@ -105,8 +116,8 @@ void OutputPainterStrategy::init( const Header *header )
 
     // Calculate how much the painter should be resized to fill the
     // outputSize with output.
-    qreal  scaleX = qreal( m_outputSize.width() )  / outputSize.width();
-    qreal  scaleY = qreal( m_outputSize.height() ) / outputSize.height();
+    qreal  scaleX = qreal( m_outputSize.width() )  / headerBoundsSize.width();
+    qreal  scaleY = qreal( m_outputSize.height() ) / headerBoundsSize.height();
     if ( m_keepAspectRatio ) {
         // Use the smaller value so that we don't get an overflow in
         // any direction.
@@ -119,15 +130,16 @@ void OutputPainterStrategy::init( const Header *header )
     kDebug(31000) << "scale = " << scaleX << ", " << scaleY;
 #endif
 
-    // Transform the EMF object so that it fits in the shape.
+    // Transform the EMF object so that it fits in the shape.  The
+    // topleft of the EMF will be the top left of the shape.
     m_painter->scale( scaleX, scaleY );
     m_painter->translate(-header->bounds().left(), -header->bounds().top());
 
-    // Calculate translation if we should center the Emf in the
+    // Calculate translation if we should center the EMF in the
     // area and keep the aspect ratio.
     if ( m_keepAspectRatio ) {
-        m_painter->translate((m_outputSize.width() - outputSize.width() * scaleX) / 2,
-                             (m_outputSize.height() - outputSize.height() * scaleY) / 2);
+        m_painter->translate((m_outputSize.width() / scaleX - headerBoundsSize.width()) / 2,
+                             (m_outputSize.height() / scaleY - headerBoundsSize.height()) / 2);
     }
 
 #if DEBUG_EMFPAINT
@@ -139,6 +151,11 @@ void OutputPainterStrategy::cleanup( const Header *header )
 {
     Q_UNUSED( header );
 
+#if DEBUG_EMFPAINT
+    if (m_painterSaves > 0)
+        kDebug(33100) << "WARNING: UNRESTORED DC's:" << m_painterSaves;
+#endif
+
     // Restore all the save()s that were done during the processing.
     for (int i = 0; i < m_painterSaves; ++i)
         m_painter->restore();
@@ -177,7 +194,7 @@ QImage* OutputPainterStrategy::image()
     return m_image;
 }
 
-   
+
 void OutputPainterStrategy::beginPath()
 {
 #if DEBUG_EMFPAINT
@@ -224,13 +241,15 @@ void OutputPainterStrategy::restoreDC( const qint32 savedDC )
     kDebug(31000) << savedDC;
 #endif
 
-    for (int i = 0; i < savedDC; ++i) {
+    // Note that savedDC is always negative
+    for (int i = 0; i < -savedDC; ++i) {
         if (m_painterSaves > 0) {
             m_painter->restore();
             --m_painterSaves;
         }
         else {
-            kDebug(33100) << "restoreDC(): try to restore painter without save";
+            kDebug(33100) << "restoreDC(): try to restore painter without save" << savedDC - i;
+            break;
         }
     }
 }
@@ -337,7 +356,7 @@ void OutputPainterStrategy::createPen( quint32 ihPen, quint32 penStyle, quint32
         kDebug(33100) << "unexpected pen type, using SolidLine" << (penStyle & 0xF);
         pen.setStyle( Qt::SolidLine );
     }
-   
+
     switch ( penStyle & PS_ENDCAP_FLAT ) {
     case PS_ENDCAP_ROUND:
         pen.setCapStyle( Qt::RoundCap );
@@ -357,10 +376,10 @@ void OutputPainterStrategy::createPen( quint32 ihPen, quint32 penStyle, quint32
     m_objectTable.insert( ihPen,  pen );
 }
 
-void OutputPainterStrategy:: createBrushIndirect( quint32 ihBrush, quint32 brushStyle, 
-                                                  quint8 red, quint8 green, quint8 blue,
-                                                  quint8 reserved,
-                                                  quint32 brushHatch )
+void OutputPainterStrategy::createBrushIndirect( quint32 ihBrush, quint32 brushStyle,
+                                                 quint8 red, quint8 green, quint8 blue,
+                                                 quint8 reserved,
+                                                 quint32 brushHatch )
 {
     Q_UNUSED( reserved );
     Q_UNUSED( brushHatch );
@@ -414,6 +433,16 @@ void OutputPainterStrategy:: createBrushIndirect( quint32 ihBrush, quint32 brush
     m_objectTable.insert( ihBrush, brush );
 }
 
+void OutputPainterStrategy::createMonoBrush( quint32 ihBrush, Bitmap *bitmap )
+{
+
+    QImage  pattern(bitmap->image());
+    QBrush  brush(pattern);
+
+    m_objectTable.insert( ihBrush, brush );
+}
+
+
 void OutputPainterStrategy::extCreateFontIndirectW( const ExtCreateFontIndirectWRecord &extCreateFontIndirectW )
 {
     QFont font( extCreateFontIndirectW.fontFace() );
@@ -505,6 +534,7 @@ void OutputPainterStrategy::selectStockObject( const quint32 ihObject )
 
 void OutputPainterStrategy::selectObject( const quint32 ihObject )
 {
+    kDebug(33100) << hex << ihObject << dec;
     if ( ihObject & 0x80000000 ) {
 	selectStockObject( ihObject );
     } else {
@@ -558,7 +588,7 @@ void OutputPainterStrategy::chord( const QRect &box, const QPoint &start, const
     qreal endAngle   = angleFromArc( centrePoint, end );
     qreal spanAngle  = angularSpan( startAngle, endAngle );
 
-    m_painter->drawChord( box, startAngle*16, spanAngle*16 ); 
+    m_painter->drawChord( box, startAngle*16, spanAngle*16 );
 }
 
 void OutputPainterStrategy::pie( const QRect &box, const QPoint &start, const QPoint &end )
@@ -691,18 +721,18 @@ void OutputPainterStrategy::modifyWorldTransform( const quint32 mode, float M11,
     kDebug(31000) << mode << M11 << M12 << M21 << M22 << Dx << Dy;
 #endif
 
-    QMatrix matrix( M11, M12, M21, M22, Dx, Dy);
+    QTransform matrix( M11, M12, M21, M22, Dx, Dy);
 
     if ( mode == MWT_IDENTITY ) {
-        m_painter->setWorldMatrix( QMatrix() );
+        m_painter->setWorldTransform( QTransform() );
     } else if ( mode == MWT_LEFTMULTIPLY ) {
-	m_painter->setWorldMatrix( matrix, true );
+	m_painter->setWorldTransform( matrix, true );
     } else if ( mode == MWT_RIGHTMULTIPLY ) {
-        QMatrix currentMatrix = m_painter->worldMatrix();
-        QMatrix newMatrix = currentMatrix * matrix;
-        m_painter->setWorldMatrix( newMatrix );
+        QTransform currentMatrix = m_painter->worldTransform();
+        QTransform newMatrix = currentMatrix * matrix;
+        m_painter->setWorldTransform( newMatrix );
     } else if ( mode == MWT_SET ) {
-	m_painter->setWorldMatrix( matrix );
+	m_painter->setWorldTransform( matrix );
     } else {
 	qWarning() << "Unimplemented transform mode" << mode;
     }
@@ -715,9 +745,9 @@ void OutputPainterStrategy::setWorldTransform( float M11, float M12, float M21,
     kDebug(31000) << M11 << M12 << M21 << M22 << Dx << Dy;
 #endif
 
-    QMatrix matrix( M11, M12, M21, M22, Dx, Dy);
+    QTransform matrix( M11, M12, M21, M22, Dx, Dy);
 
-    m_painter->setWorldMatrix( matrix );
+    m_painter->setWorldTransform( matrix );
 }
 
 void OutputPainterStrategy::extTextOutA( const ExtTextOutARecord &extTextOutA )
@@ -730,7 +760,8 @@ void OutputPainterStrategy::extTextOutA( const ExtTextOutARecord &extTextOutA )
 
     m_painter->setPen( m_textPen );
 
-    QPoint position = extTextOutA.referencePoint();
+    // Set position to the reference point. It's assumed that this is the baseline.
+    QPoint       position = extTextOutA.referencePoint();
     QFontMetrics fontMetrics = m_painter->fontMetrics();
     switch ( m_textAlignMode & TA_VERTMASK ) {
         case TA_TOP:
@@ -748,10 +779,11 @@ void OutputPainterStrategy::extTextOutA( const ExtTextOutARecord &extTextOutA )
     // TODO: Handle the rest of the test alignment mode flags
 
     m_painter->drawText( position, extTextOutA.textString() );
-
-    kDebug(33100) << "extTextOutA: ref.point = " 
+#if DEBUG_EMFPAINT
+    kDebug(33100) << "extTextOutA: ref.point = "
                   << extTextOutA.referencePoint().x() << extTextOutA.referencePoint().y()
                   << ", Text = " << extTextOutA.textString().toLatin1().data();
+#endif
 
     m_painter->restore();
 }
@@ -768,6 +800,7 @@ void OutputPainterStrategy::extTextOutW( const QPoint &referencePoint, const QSt
     // The TA_UPDATECP flag tells us to use the current position
     if (m_textAlignMode & TA_UPDATECP) {
         // (left, top) position = current logical position
+        kDebug(31000) << "TA_UPDATECP";
         x = m_currentCoords.x();
         y = m_currentCoords.y();
     }
@@ -782,25 +815,23 @@ void OutputPainterStrategy::extTextOutW( const QPoint &referencePoint, const QSt
     else if ((m_textAlignMode & TA_RIGHT) == TA_RIGHT)
         x -= width;
 
-    // Vertical align. 
+    // Vertical align.
     if ((m_textAlignMode & TA_BASELINE) == TA_BASELINE)
         y -= fm.ascent();  // (height - fm.descent()) is used in qwmf.  This should be the same.
     else if ((m_textAlignMode & TA_BOTTOM) == TA_BOTTOM) {
         y -= height;
+    }
 
 #if DEBUG_EMFPAINT
-        kDebug(31000) << "font = " << m_painter->font() << " pointSize = " << m_painter->font().pointSize()
-                      << "ascent = " << fm.ascent() << " height = " << fm.height()
-                      << "leading = " << fm.leading();
+    kDebug(31000) << "font = " << m_painter->font() << " pointSize = " << m_painter->font().pointSize()
+                  << "ascent = " << fm.ascent() << " height = " << fm.height()
+                  << "leading = " << fm.leading();
 #endif
-    }
 
     // Use the special pen defined by mTextPen for text.
     QPen  savePen = m_painter->pen();
     m_painter->setPen(m_textPen);
-
     m_painter->drawText(x, y, width, height, Qt::AlignLeft|Qt::AlignTop, text);
-
     m_painter->setPen(savePen);
 }
 
@@ -991,7 +1022,7 @@ void OutputPainterStrategy::setClipPath( const quint32 regionMode )
         m_painter->setClipPath( *m_path, Qt::ReplaceClip );
         break;
     default:
-        qWarning() <<  "Unexpected / unsupported clip region mode:" << regionMode; 
+        qWarning() <<  "Unexpected / unsupported clip region mode:" << regionMode;
         Q_ASSERT( 0 );
     }
 }
@@ -1050,9 +1081,9 @@ void OutputPainterStrategy::stretchDiBits( StretchDiBitsRecord &record )
 {
 #if DEBUG_EMFPAINT
     kDebug(31000) << "Bounds:    " << record.bounds();
-    kDebug(31000) << "Dest rect: " 
+    kDebug(31000) << "Dest rect: "
                   << record.xDest() << record.yDest() << record.cxDest() << record.cyDest();
-    kDebug(31000) << "Src rect:  " 
+    kDebug(31000) << "Src rect:  "
                   << record.xSrc() << record.ySrc() << record.cxSrc() << record.cySrc();
     kDebug(31000) << "Raster op: " << hex << record.rasterOperation() << dec;
                   //<< record.bkColorSrc();
@@ -1113,13 +1144,15 @@ void OutputPainterStrategy::stretchDiBits( StretchDiBitsRecord &record )
         // instead of logical coordinates like all the other types of
         // records.  Therefore we have to rescale the target from
         // physical to logical coordinates.
+#if DEBUG_EMFPAINT
         qreal scaleX = qreal(m_header->frame().width()) / qreal(m_header->bounds().width());
         qreal scaleY = qreal(m_header->frame().height()) / qreal(m_header->bounds().height());
-#if DEBUG_EMFPAINT
         kDebug(31000) << "Scale = " << scaleX << scaleY;
 #endif
-        QRectF realTarget(QPoint(target.x() / scaleX, target.y() / scaleY),
-                          QSize(target.width() / scaleX, target.height() / scaleY));
+
+        QRectF realTarget(QPoint(target.x(), target.y()),
+                          QSize(target.width(), target.height()));
+
 #if DEBUG_EMFPAINT
         kDebug(31000) << "    realTarget" << realTarget;
 #endif
@@ -1143,11 +1176,11 @@ qreal OutputPainterStrategy::angleFromArc( const QPoint &centrePoint, const QPoi
 qreal OutputPainterStrategy::angularSpan( const qreal startAngle, const qreal endAngle )
 {
     qreal spanAngle = endAngle - startAngle;
-    
+
     if ( spanAngle <= 0 ) {
         spanAngle += 360;
     }
-    
+
     return spanAngle;
 }
 
